""" Analysis of the terms of three-particle (iL)^n in three dimensions """

from itertools import combinations
import numpy as np
from mpliouville import MPLPDR, get_stats_expr, get_stats_repl
from systems import MPPS3D
import yaml

nprtcls = 3
numeric = {'tool': 'subs'}

symbols = [str(i)+'x:z' for i in range(nprtcls)]
indic = list(range(3*nprtcls))
pairs = list(combinations(zip(indic[0::3], indic[1::3], indic[2::3]), 2))
ffparams = [{'class': 'FFLennardJones', 'pair': p, 'params': {}} for p in pairs]
masses = np.full((nprtcls*3, ), 1)
qval = np.full((nprtcls*3, ), 1) # these are not valid
pval = np.full((nprtcls*3, ), 0)
system = MPPS3D(ffparams, symbols, masses, qval, pval, numeric=numeric)

nterms = 3

iln = MPLPDR(nterms, system, numeric=numeric)
print(yaml.dump(get_stats_expr(iln.ilnqexpr, details=True)))
print(yaml.dump(get_stats_expr(iln.ilnpexpr, details=True)))
print(yaml.dump(get_stats_repl(iln.repl, details=True)))
print(iln.get_val_float(qval, pval))
