""" Scalability measurement of the construction of (iL)^n f expressions for a
    N-particle 3D system """

from itertools import combinations
import numpy as np
from mpliouville import MPLPQP, MPLPAQ, MPLPVQ, MPLPDR, MPLPDV
from mpliouville import get_stats_expr, get_stats_repl
from pandas.io.json import json_normalize
from systems import MPPS3D
import time

def stats_summary(mplpobj):
    stats = {}
    stats['ilnqexpr'] = get_stats_expr(mplpobj.ilnqexpr)
    stats['ilnpexpr'] = get_stats_expr(mplpobj.ilnpexpr)
    if hasattr(mplpobj, 'repl'):
        stats['ilnrepl'] = get_stats_repl(mplpobj.repl)
    syms = sum(stats[key]['symbols'] for key in stats)
    adds = sum(stats[key]['adds'] for key in stats)
    muls = sum(stats[key]['multiplies'] for key in stats)
    ops = sum(stats[key]['operations'] for key in stats)
    stats['totals'] = {'symbols': syms, 'adds': adds, 'mutiplies': muls,
                       'storage': syms + adds + muls, 'operations': ops}
    return stats

numeric = {'tool': 'subs'}

statistics = []
for nprtcls in range(2, 5):
    print('nprtcls:', nprtcls)
    symbols = [str(i)+'x:z' for i in range(nprtcls)]
    indic = list(range(3*nprtcls))
    pairs = list(combinations(zip(indic[0::3], indic[1::3], indic[2::3]), 2))
    ffparams = [{'class': 'FFLennardJones', 'pair': p, 'params': {}} for p in pairs]
    masses = np.full((nprtcls*3, ), 1)
    qval = np.full((nprtcls*3, ), 1)
    pval = np.full((nprtcls*3, ), 0)
    for nterms in range(1, 6):
        print('nterms:', nterms)
        for mplp in (MPLPQP, MPLPAQ, MPLPVQ, MPLPDR, MPLPDV):
            system = MPPS3D(ffparams, symbols, masses, qval, pval,
                            numeric=numeric)
            print('class:', mplp.__name__)
            ts = time.time()
            mplpobj = mplp(nterms, system, numeric=numeric)
            te = time.time()
            stats = stats_summary(mplpobj)
            stats['class'] = mplp.__name__
            stats['variant'] = 'default'
            stats['nterms'] = nterms
            stats['nparticles'] = len(symbols)
            stats['preparation time'] = te - ts
            statistics.append(stats)

        for mplp, var in zip((MPLPDR, MPLPDV), ('r', 'r')):
            system = MPPS3D(ffparams, symbols, masses, qval, pval,
                            numeric=numeric)
            print('class:', mplp.__name__)
            ts = time.time()
            mplpobj = mplp(nterms, system, numeric=numeric, variant=var)
            te = time.time()
            stats = stats_summary(mplpobj)
            stats['class'] = mplp.__name__
            stats['variant'] = var
            stats['nterms'] = nterms
            stats['nparticles'] = len(symbols)
            stats['preparation time'] = te - ts
            statistics.append(stats)

df = json_normalize(statistics)
df.to_json('exprstats.json')
